\ MPE 80x96 Single Chip Forth Code definitions

((
Copyright (c) 1990-1991
MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF

tel: (+44) 703 631441
fax: (+44) 703 339691

March 1990	Original issue:

May 1991	converted to text file
		converted to v2.0 Umbilical Forth

October 1991	8096 version
))
\ dodoes docreate next execute lit ?branch branch (of)

ASSEMBLER
l: dodoes
  sub rp # 2   st ip rp []              \ put IP on R. stack
  pop ip                                \ get new IP
l: docreate
  pop ax                                \ get new PFA
  push tos                              \ save old TOS
  ld tos ax                             \ form new TOS
l: next
  ld ax ip []+                          \ fetch CFA, incr. IP
  br ax []                              \ jmp to it
FORTH

CODE EXECUTE	\ cfa -- ; execute CFA on stack
  ld ax tos   pop tos   br ax []   END-CODE

CODE LIT	\ -- n ; for in-line literal
  push tos   ld tos ip []+
  next,   end-code

CODE BRANCH     \ -- ; target address is in-line word
L: BRAN1
  ld ip ip []   next,   end-code

CODE ?BRANCH	\ n -- ; branch if n=0
  cmp tos r0   pop tos   je bran1
L: 0BRAN1
  add ip # 2   next,     END-CODE

CODE (OF)       \ n1 n2 -- ; n1 n2 -- n1 [branch taken]
  cmp tos sp []   pop tos   jne bran1   \ compare n1 n2, drop n2
  pop tos                               \ equal, drop n1 too
  add ip # 2   next,                    \ skip branch offset
end-code
\ (loop) (+loop) (do) (?do) i

CODE (LOOP)     \ -- ; affects return stack when loop ends
  ld ax rp []   inc ax   jv loop-exit   \ test index-limit+8000
L: LOOP-CONT
  st ax rp []   ld ip ip []             \ round again
  next,   end-code

CODE (+LOOP)    \ n -- ; see (LOOP)
  add ax tos & rp []  pop tos           \ add increment
  jnv loop-cont                         \ check, again if NV
L: LOOP-EXIT
  add rp # 6                            \ update RP
  add ip # 2   next,   end-code         \ skip branch offset

hex

CODE (DO)       \ limit start --
l: pdo1
  sub rp # 6                            \ 3 new items on stac
  ld ax ip []+   st ax 4 rp ,[]         \ stash end address
  pop ax   add ax # 08000   st ax 2 rp ,[] \ save limit+08000h
  sub tos ax   st tos rp []             \ start-limit-08000h
  pop tos                               \ restore stack
  next,  end-code

CODE (?DO)      \ limit start --
  cmp tos sp []   jne pdo1              \ do loop if different
  add sp # 2   pop tos                  \ 2drop, maintain TOS
  ld ip ip []   next,   end-code        \ take branch

code i		\ -- n ; return DO ... LOOP index
  push tos
  ld tos rp []                          \ start-limit-08000+inc
  add tos 2 rp ,[]                      \ limit+08000
  next,   end-code

\ min max

code min        \ n1 n2 -- n1|n2 ; return lesser of n1 and n2
  cmp tos sp []   jge minx              \ tos-nos
  add sp # 2   next,                    \ use TOS
l: minx
  pop tos   next,   end-code            \ use NOS

code max        \ n2 n1 -- n1|n2 ; return greater of n1 and n2
  cmp tos sp []   jle maxx              \ tos-nos
  add sp # 2   next,                    \ use TOS
l: maxx
  pop tos   next,   end-code            \ use NOS

\ um* * m* um/mod ( sm/mod ) m/mod /mod

code um*        \ u1 u2 -- ud ; unsigned multiply
  mulu tos sp []
  st tos sp []   ld tos tosext
  next,   end-code

code *          \ n1 n2 -- n3 ; signed multiply
  mul tos sp []+   next,   end-code

code m*         \ n1 n2 -- d
  mul tos sp []
  st tos sp []   ld tos tosext
  next,   end-code

hex

code um/mod     \ u32 u16 -- urem uquot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  divu tos bx				\ perform division
  jv um/mod-oflow			\ overflow check
  push tosext				\ save remainder, quotient in TOS
  next,
l: um/mod-oflow
  ld tos # 0FFFF   push tos
  next,  end-code
((
code sm/mod     \ n32 n16 -- rem quot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  div tos bx				\ perform division
  jv sm/mod-oflow
  push tosext
  next,
l: sm/mod-oflow
  ld tos # 07FFF   push tos
  next,  end-code
))
((
code m/mod	\ n32 n16 -- rem quot
  ld bx tos                             \ save divisor
  pop tosext   pop tos                  \ get dividend
  div tos bx				\ perform division
  jv m/mod-oflow			\ overflow check
  or tos tos				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  dec tos		\ decrement quotient
                    add tosext bx	\ add divisor to remainder
	       endif,
      endif,
  push tosext				\ save remainder
  next,
l: m/mod-oflow
  ld tos # 07FFF   push tos
  next,  end-code

: /mod		\ n1 n2 -- rem quot
  >r s>d r> m/mod   ;
))
\ / u/ mod */mod */ mu/mod

code /		\ n1 n2 -- quot
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  ld ax bx   xor ax tosext		\ form sign of quotient
  div tos bx				\ perform division
  jv /-oflow				\ overflow check
  or ax ax				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  dec tos  endif,	\ decrement quotient
      endif,
  next,
l: /-oflow
  ld tos # 07FFF
  next,  end-code

code u/		\ u1 u2 -- uquot
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  divu tos bx				\ perform division
  jv u/-oflow				\ overflow check
  next,
l: u/-oflow
  ld tos # 0FFFF
  next,  end-code

code mod	\ n1 n2 -- rem
  ld bx tos                             \ save divisor
  pop tos   ext tos			\ get dividend
  div tos bx				\ perform division
  jv mod-oflow				\ overflow check
  or tos tos				\ check sign of quotient
  lt, if,  or tosext tosext		\ if remainder exists
	   ne, if,  add tosext bx	\ add divisor to remainder
	       endif,
      endif,
  ld tos tosext				\ save remainder
  next,
l: mod-oflow
  ld tos # 07FFF
  next,  end-code
((
: */mod		\ n1 n2 n3 -- rem quot ; n1*n2/n3 with 32-bit intermediate
  >r m* r> m/mod  ;

: */		\ n1 n2 n3 -- quot ; n1*n2/n3 with 32-bit intermediate
  */mod nip  ;
))
: mu/mod	\ ud n -- urem udquot ; used in number conversion
  >r 0 r@ um/mod r> swap >r um/mod r> ;

\ and or xor not

CODE AND        \ n1 n2 -- n3
  and tos sp []+   next,   end-code

CODE OR         \ n1 n2 -- n3
  or tos sp []+   next,   end-code

CODE XOR        \ n1 n2 -- n3
  xor tos sp []+   next,   end-code

CODE NOT        \ n1 -- n2
  not tos   next,   end-code


\ sp@ sp! rp@ rp! exit >r r> r@

CODE SP@        \ -- n ; return data stack pointer
  push tos   ld tos sp   next,   END-CODE

CODE SP!        \ n -- ; set data stack pointer
  ld sp tos   pop tos   next,   end-code

CODE RP@        \ -- n ; get return stack pointer
  push tos   ld tos rp   next,   end-code

CODE RP!        \ n -- ; set return stack pointer
  ld rp tos   pop tos   next,   end-code

code exit	\ -- ; return from colon definition
  ld ip rp []+   next,   end-code

CODE >R         \ n -- ; copy to return stack
  sub rp # 2   st tos rp []   pop tos
  next,   end-code

CODE R>         \ -- n ; from return stack
  push tos   ld tos rp []+
  next,   end-code

CODE R@         \ -- n ; copy of return stack
  push tos   ld tos rp []   next,   end-code
\ 0= 0<> = <> 0< 0> u< u> < >

CODE 0<		\ n -- t/f
  cmp tos 0   jlt set-tru   sjmp set-fls   end-code

CODE 0>		\ n -- t/f
  cmp tos 0   jgt set-tru   sjmp set-fls   end-code

CODE 0<>	\ n -- t/f
  cmp tos 0   jne set-tru   next,   end-code

CODE 0=		\ n -- t/f
  cmp tos r0   jne set-fls
L: SET-TRU
  ld tos # -1   next,
L: SET-FLS
  clr tos   next,   END-CODE

CODE =		\ n1 n2 -- t/f
  cmp tos sp []+   je set-tru   sjmp set-fls   end-code

CODE <>		\ n1 n2 -- t/f
  cmp tos sp []+   jne set-tru   sjmp set-fls   end-code

CODE U<		\ n1 n2 -- t/f
  cmp tos sp []+   jh set-tru   sjmp set-fls   end-code

CODE U>		\ n1 n2 -- t/f
  cmp tos sp []+   jh set-fls   je set-fls   sjmp set-tru
  end-code

CODE <          \ n1 n2 -- t/f
  cmp tos sp []+   jgt set-tru   sjmp set-fls  end-code

CODE >          \ n1 n2 -- t/f
  cmp tos sp []+   jlt set-tru   sjmp set-fls   end-code
\ d< d> d0= d=

((
CODE D<         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jgt d<tru   jlt d<fls
  cmp tosext sp []  jgt d<tru
l: d<fls
  add sp # 2   clr tos   next,
l: d<tru
  add sp # 2   ld tos # -1   next,
  end-code

code d>         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jlt d<tru   jgt d<fls
  cmp tosext sp []   jlt d<tru   sjmp d<fls
end-code
))

((
code d=         \ d1 d2 -- t/f
  pop tosext
  cmp tos sp []+   jne d<fls            \ false if different
  cmp tosext sp []   jne d<fls          \ false if different
  sjmp d<tru                            \ true if same!
end-code

code d0=        \ d -- t/f
  or tos sp []+
  ne, if,   clr tos   next,   endif,
  ld tos # -1   next,   end-code
))
\ negate ?negate abs dnegate ?dnegate dabs

CODE negate     \ n1 -- -n1
l: neg1
  neg tos   next,   end-code

code ?negate    \ n1 t/f -- n1/-n1
  cmp tos r0   pop tos   jlt neg1
  next,   end-code

code abs        \ n1 --  |n1|
  cmp tos r0   jlt neg1
  next,   end-code

CODE dnegate    \ d1 -- -d1
l: dneg1
  ld tosext sp []   not tos   neg tosext
  e, if,  inc tos  endif,
  st tosext sp []   next,   end-code

code ?dnegate   \ d1 t/f -- d1/-d1
  cmp tos r0   pop tos   jlt dneg1
  next,   end-code

code dabs       \ d1 -- |d1|
  cmp tos r0   jlt dneg1
  next,   end-code

\ over 2over drop 2drop swap ( 2swap ) dup ?dup 2dup

CODE OVER       \ n1 n2 -- n1 n2 n1
  push tos   ld tos 2 sp ,[]
  next,   end-code

CODE 2OVER      \ n1 n2 n3 n4 -- n1 n2 n3 n4 n1 n2
  push tos   ld tos 6 sp ,[]
  push tos   ld tos 6 sp ,[]
  next,   end-code

CODE DROP       \ n1 --
  pop tos   next,   end-code

CODE 2DROP      \ n1 n2 --
  add sp # 2   pop tos   next,   end-code

CODE SWAP       \ n1 n2 -- n2 n1
  ld tosext sp []   st tos sp []
  ld tos tosext   next,   end-code
((
CODE 2SWAP      \ n1 n2 n3 n4 -- n3 n4 n1 n2
  pop tosext
  pop ax   pop bx
  push tosext  push tos
  push bx   ld tos ax
  next,   end-code
))
CODE DUP        \ n1 -- n1 n1
l: dup1
  push tos   next,   end-code

CODE 2DUP       \ n1 n2 -- n1 n2 n1 n2
  ld tosext sp []
  push tos   push tosext
  next,   end-code

code ?dup       \ n1 -- n1 [n1]
  cmp tos r0   jne dup1
  next,   end-code
\ rot -rot pick ( roll ) nip tuck

CODE ROT        \ n1 n2 n3 -- n2 n3 n1
  pop ax   pop bx
  push ax   push tos   ld tos bx
  next,   end-code

code -rot       \ n1 n2 n3 -- n3 n1 n2
  pop ax   pop bx
  push tos   push bx   ld tos ax
  next,   end-code

code pick       \ item -- n
  shl tos # 1   add bx sp & tos
  ld tos bx []
  next,   end-code
((
: ROLL          \ n1 n2 .. nk n -- n2 .. nk n1
		\ DO NOT USE THIS WORD UNLESS YOU ABSOLUTELY HAVE TO !
  >R R@ PICK
  SP@ DUP 2+   R> 1+ 2* CMOVE>  DROP  ;
))
code nip        \ n1 n2 -- n2
  add sp # 2   next,   end-code

code tuck       \ n1 n2 -- n2 n1 n2
  pop ax   push tos   push ax
  next,   end-code
\ c@ @ ( 2@ )  c! ! ( 2! ) +!  1+ 2+ 1- 2-  2* 2/ u2/

CODE C@         \ addr -- b ; byte fetch
  ldbze tos tos []
  next,   end-code

CODE @          \ addr -- n ; word fetch
  ld tos tos []    next,   end-code
((
CODE 2@         \ addr -- d ; double word fetch
  ld ax 2 tos ,[]   push ax
  ld tos tos []
  next,   end-code
))

CODE C!         \ b addr -- ; byte store
  pop ax   stb al tos []   pop tos
  next,   end-code

CODE !          \ n addr -- ; word store
  pop tos []   pop tos
  next,   end-code
((
CODE 2!         \ d addr -- ; double word store
  pop tos []   pop 2 tos ,[]
  pop tos   next,   end-code
))

CODE C+!	\ n addr --
  pop ax   addb al tos []   stb al tos []
  pop tos   next,   end-code
CODE +!         \ n addr --
  pop ax   add ax tos []   st ax tos []
  pop tos   next,   end-code

CODE 1+         \ n -- n+1
  inc tos   next,   end-code
CODE 2+         \ n -- n+2
  add tos # 2   next,   end-code

code 1-         \ n -- n-1
  dec tos   next,   end-code
code 2-         \ n -- n-2
  sub tos # 2   next,   end-code

CODE 2*         \ n1 -- n2
  shl tos # 1   next,   end-code
CODE 2/         \ n1 -- n2
  shra tos # 1   next,   end-code
CODE u2/         \ n1 -- n2
  shr tos # 1   next,   end-code
\ + - d+ d-

CODE +          \ n1 n2 n1+n2
  add tos sp []+   next,   end-code

CODE -          \ n1 n2 -- n1-n2
  pop ax   sub tos ax & tos
  next,   end-code

CODE D+         \ d1 d2 -- d3
  ld tosext sp []+                      \ get d2.l
  add tosext 2 sp ,[]                   \ d3.l=d2.l+d1.l
  addc tos sp []+                       \ d3.h=d2.h+d1.h
  st tosext sp []                       \ stash d3.l
  next,   END-CODE

CODE D-         \ d1 d2 -- d1-d2
  ld tosext sp []+                      \ d2.l
  ld ax sp []+   ld bx sp []            \ ax=d1.h bx=d1.l
  sub bx tosext   subc ax tos           \ perform subtraction
  st ax tos   st bx sp []               \ stash result
  next,   end-code                      \
\ s->d within? on off count bounds

CODE S>D        \ n -- d
  push tos   ext tos   ld tos tosext
  next,   end-code

code within?    \ x a b -- t/f ; true if a<=x<=b
  pop tosext   pop ax
  cmp ax tosext   jlt wit1
  cmp ax tos   jgt wit1
  ld tos # -1   next,
l: wit1
  clr tos   next,   end-code

code on         \ addr --
  ld ax # -1   st ax tos []   pop tos   next,   end-code

code off        \ addr --
  st r0 tos []   pop tos   next,   end-code

code count      \ addr -- addr+1 len
  clrb ah   ldb al tos []+
  push tos   ld tos ax   next,   end-code

code bounds     \ addr len -- addr+len addr
  ld ax sp []   add tos ax   st tos sp []   ld tos ax
  next,   end-code
\ : variable constant user

assembler
here is-action-of :
  sub rp # 2   st ip rp []              \ save IP on R. stack
  pop ip                                \ get new IP
  next,					\ go to it
forth

assembler
here is-action-of variable
here is-action-of constant
  pop ax				\ pop PFA
  push tos				\ save old TOS
  ld tos ax []				\ get address of data
  next,
forth

assembler
here is-action-of user
  pop ax				\ pop PFA
  push tos				\ save old TOS
  add tos up & ax []			\ add base address & offset
  next,
forth
\ all done 

decimal

;p
