\ Timer1 example interrupt code
\ =============================
\
\ This code released as example code only, and id for use
\ with the MPE 80c196 cross-compiler.
\ (c) MicroProcessor Engineering Ltd., 1991
\ 133 Hill Lane
\ Southampton
\ SO1 5AF
\
\ Phone: (+44) 703 631441
\ Fax:   (+44) 703 339691
\
\
\ Author: J. E. Holt
\ Date: 2/10/91
\ 
\ The timer1 interrupt is enabled and serviced by an ISR which simply 
\ sets the variable 'ticked' and reloads the timer. The main level program
\ executes the loop, then waits for the clock to tick. Should the clock
\ tick before the loop execution is complete, an error routine is entered
\
\ N.B. since the code reloads timer1, the HSO and HSI units may not work 
\ correctly. Before entering the error handler a global interrupt disable
\ is performed
\
\ Entry points
\      main                                 n --
\      starts the system with cycle time of n timer1 increments.
\
\ loop-init must be assigned to perform loop initialisation operations;
\
\     assign <loop-init-code> to-do loop-init
\
\ loop-action must be assigned to perform the loop operations:
\
\     assign <loop-code-word> to-do loop-action
\
\ error-action must be assigned to an error handler to be invoked 
\ in the event of a timing error ( i.e. if loop overruns 1 cycle time
\
\     assign <error-handle-code> to-do error-action

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  constants and variables

HEX

\ addresses of 80C196 SFRs

014 constant WSR
0A  constant timer1
016 constant IOC1
08  constant int-mask

variable interval		\ number used to load timer1
variable ticked			\ set by ISR, reset by main level

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  set-clock

: set-clock		\ interval -- ; load timer with interval
  0F WSR c!			\ switch to hwindow 15
  timer1 !			\ load timer 
  0 WSR c!			\ switch back to hwindow 0
;

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  init-clock

: init-clock		\ interval -- ; initialisation routine
  negate			\ convert interval to form for loading
				\ ( timer1 increments, overflowing at
				\ 65526 )
  dup interval !		\ store in variable
  1 int-mask set-bit            \ enables timer1 interrupt
  4 IOC1 set-bit                \ timer1 to interrupt on overflow
  ei				\ global interrupt enable
  set-clock			\ load timer with interval
;

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  clock-isr

: clock-isr		\ -- ; interrupt handler
  -1 ticked !			\ set variable 'ticked'
  interval @                    \ 
  set-clock			\ reload timer1
;

\ attach clock-isr to interrupt vector 02000

l: timer1-int
 ] clock-isr reti [                     \ action, return

ASSEMBLER                               \
l: timer1-ivec                          \ registers saved by IRQ
  pushf   push ip                       \ save PSW and IP
  ld ip # timer1-int			\ load new IP
  ljmp next-int				\ join common code
FORTH

timer1-ivec 02000 !                     \ set vector

\ -----End of Page-----
\ -- R.G.G -- 3/12/91 --  reset-int

: reset-int		\ -- ; turn off the interrupts
  1 int-mask reset-bit	        \ disable timer1 interrupts
  4 IOC1 reset-bit		\ turn off timer1 interrupts
  di				\ global interrupt disable
;

defer loop-init               	\ loop initialisation code defined elsewhere
defer loop-action		\ loop code defined elsewhere
defer error-action		\ error handler defined elsewhere

: main			\ interval -- ; entry point
  loop-init                             \ loop initialisation code goes here
  init-clock				\ initialise clock etc.
  0 ticked !                            \ ensure 'ticked' is reset at start
  begin
    loop-action                         \ main loop code goes here
    ticked @ 	                        \ check if clock has ticked
    if 	                                \ if so, then
       reset-int			\ shut down interrupts and
       error-action			\ jump to error handler
    else				\ otherwise
       begin 
          ticked @ 
       until		                \ await clock tick
       0 ticked !			\ reset variable to await next
    then
  again
;
