\ Code to demonstrate inputs and outputs on the multi-function i/o baord 

\ (C) MicroProcessor Engineering Ltd
\ 133 Hill Lane
\ Southampton
\ SO1 5AF 

\ Phone	 0703 631441
\ Fax	 0703 339691
\ E-mail mpe@cix.compulink.co.uk					

\ Read and write I/O ports

((
Routine to read 4 switches attached to pins IN4, IN5, IN6, and IN7 (J3A)
on the MPE Multifunction I/O board. The values of these pins are then 
applied as outputs to drive some LEDs attached to pins Q4 - Q7 (J2B) on
the same board. The effect of this is that when any of the switches are 
closed the corresponding LED will light. IN4 lights Q4, IN5 lights Q5 etc
))

: test-task	\ -- ; read inputs and pass masked value to outputs			   
  0			\ start off by writing 0 (turn off op) 
  begin	    
    0 iob		\ write last value read and read new value (board 0)
    240 and		\ mask top 4 bits of val read, so val can be written
    pause		\ this will multi task.
  again
; 

: run-io	\ -- ; run the application
  init-multi				\ initialise
  assign test-task 1 to-task		\ assign our task
  1 activate				\ kick task off
  multi					\ run the scheduler
;

\ example interrupt service routine for High Speed Inputs		

variable lasttime			\ last time stamp that went through
variable counter			\ something to increment

: data-ready-isr	\ -- ; the ISR - increment counter
  read-hsi-pins	       			\ get the status
  drop					\ dont care what the pins are doing
  lasttime !				\ save the last time stamp for later
  1 counter +!				\ inc counter
;				

: run-hsi		\ -- ; initialise interrupts etc
  init-hsi				\ flush buffers etc
  2 set-hsi0-mode    			\ trigger on every falling edge	
					\ ie when switch is closed
  2 set-hsi1-mode    			\ trigger on every falling edge	
					\ ie when switch is closed
  assign data-ready-isr to-do hsi-isr   \ assign it to the forth high 
					\ level handler in int196.fth
;

\ Example ISR for High Speed Outputs - square wave generation
\ R.G.G -- 3/12/92

variable hso0-time			\ copy of the time set
variable hso0-status			\ copy of the level of the port
variable interval			\ Period of the square wave


: toggle-hso	\ chan# -- ; change hso state
  hso0-status @ dup
  not hso0-status !			\ toggle it
  if		 			\ 0 below means no CAM lock
    0 set-hso-t1
  else
    0 reset-hso-t1
  endif
;

: square-wave		\ -- ; interrupt routine
  interval @ hso0-time @ +		\ get the time, and calculate next one 
  dup hso0-time !			\ shadow it
  0 toggle-hso				\ set it on channel 0
;

: init-wave		\ interval -- ; set up and do
  interval !		       		\ initialise the interval variable
  08 int_mask set-bit	       		\ enable hso interrupts
  ei
  0 0 0 reset-hso-t1  			\ use reset-hso to init interrupts etc
;

: run-hso		\ -- ; set sqaure wave up and assign ISR
  init-hso
  assign square-wave to-do hso-isr
  2000 init-wave
;


