\ MPE 196 Digital I/O board handlers
\ This file contains drivers for the 8255 and the 6242 real time clock
\ on the User Interface Board (UIF196) for the MPE 80C196 PowerBoard range.

\ Copyright (C) 1992 MicroProcessor Engineering

\ MicroProcessor Engineering
\ 133 Hill Lane
\ Shirley
\ Southampton SO1 5AF
\ England

\ tel: (+44) 703 631441
\ fax: (+44) 703 339691
\ net: mpe@compulink.co.uk

\ compatibility words - delete when cross compiling, use when downloading

\ ((
: equ		\ n -- ; for compatibility with the cross compiler
  constant  ;

: allot-ram     \ n -- ; reserve space in RAM ; for compatibility with XC
  allot  ;
\ ))
\ Chip selects

hex

0F000 equ cs8255	\ 8255 parallel i/o chip A
0F010 equ csrtc		\ 6242 real time clock

\
\ identify the ports on the 8255 chips
\
cs8255 3 + equ control-8255
cs8255 2 + equ pc-8255
cs8255 1 + equ pb-8255
cs8255 0 + equ pa-8255

: init-8255	\ -- ; Mode 0, PA o/p, PB o/p, PCH i/p, PCL o/p
  08A control-8255 c!			\ first 8255
  07 pc-8255 c!  ;

\ OKI 6242B Real time clock driver code

\ RTC working buffer

hex

variable rtc-img  010 allot-ram        \ working buffer


\ rtc@ rtc! rtc-i@ rtc-i! start-rtc

: rtc@          \ reg -- n ; fetch nibble from RTC register
  csrtc + c@  0F and  ;			\ only bottom four bits needed

: rtc!          \ n reg -- ; store nibble in RTC register
  csrtc + c!  ;

: rtc-i@        \ reg -- n ; fetch byte from RAM image of RTC registers
  rtc-img  + c@  ;

: rtc-i!        \ n reg -- ; store byte in RAM image of RTC registers
  rtc-img + c!  ;

: start-rtc	\ -- ; run this after every power up of the RTC
  0D rtc@ drop		\ wake DS1210 with two memory accesses
  0D rtc@ drop
  0 0D rtc!		\ reset IRQ and HOLD
  4 0E rtc!		\ pulse every second
  7 0F rtc!		\ set stop & reset
  7 0F rtc!		\ 24 hr set
  4 0F rtc!  ;		\ remove stop & reset


\ rtc-busy@ set-hold reset-hold claim-rtc

hex

: rtc-busy@	\ -- t/f ; return rtc-busy flag, 1 means valid read
  0D rtc@  02 and  ;			\ busy is d1 in c-reg of rtc

: set-hold	\ -- ; set HOLD bit
  0D rtc@  01 or  0D rtc!  ;

: reset-hold	\ -- ; reset HOLD bit
  0D rtc@  0E and  0D rtc!  ;

: claim-rtc     \ -- ; wait for rtc to be available
  begin  set-hold		\ set hold bit
         rtc-busy@              \ check busy flag
    while  reset-hold		\ if clear, release hold
  repeat  ;

\ rd-rtc wr-rtc

hex

: rd-rtc        \ -- ; read 6242 data to buffer
  claim-rtc			\ put RTC on HOLD
  0D 0                          \ 13 items, registers 0..C
  do  i rtc@			\ get nibble
      i 5 =  if 03 and endif    \ mask out AM/PM flag
      i rtc-i!                  \ stash in memory image
  loop
  reset-hold  ;			\ release HOLD

: wr-rtc        \ addr -- ; write buffer to 6242
  claim-rtc			\ put RTC on hold
  0D 0				\ 12 items
  do  dup c@  i rtc!            \ copy byte to clock
      1+			\ increment address
  loop
  drop                          \ discard address
  reset-hold  ;			\ release HOLD

create months	\ -- addr ; holds text of months
  ", JanFebMarAprMayJunJulAugSepOctNovDec"

create dows	\ -- addr ; holds text of Days Of Week
  ", SunMonTueWedThuFriSat"


\ as-months as-dow

decimal

\ given month digit send out month as text
: as-month              ( n -- )
  1 max 12 min  1-  3 *         \ get offset into string
  months 1+ +                   \ add to start of string
  3 type  ;                     \ send three digits

: as-dow                ( n -- )
  0 max 6 min			\ 0..6, 0=Sunday, 6=Staturday
  3 * dows +  1+		\ 3 bytes/day, skip count byte
  3 type  ;

\ show-digits show-month show-dow

decimal
: show-digits	\ offset -- ; display the time digit
  dup 1+ rtc-i@ ascii 0 + emit
         rtc-i@ ascii 0 + emit  ;

: show-month	\ --
  09 rtc-i@ 10 *
  08 rtc-i@  +
  1 max 12 min
  as-month  ;

hex

: show-dow	\ --
  0c rtc-i@  07 and  as-dow  ;


\ time date                                        drm 14/09/90

hex

: time		\ -- ; show time as eg   10:34:56
  rd-rtc
  4 show-digits ." :"
  2 show-digits ." :"
  0 show-digits  ;

: date		\ -- ; show date as eg   12jun92
  rd-rtc
  show-dow  space			\ Day of week is not reliable
  06 show-digits  ." /"
  show-month  ." /"
  0A show-digits  ;

: now		\ -- ; display date and time
  date space time  ;

\ !rtc-img                                         drm 14/09/90

decimal

: !rtc-img      \ b offset -- ; store byte in nibble pair
  >r                                    \ save offset
  0 10 um/mod                           \ split digits
  r@ 1+ rtc-i!  r> rtc-i!  ;            \ stash in buffer


\ set-time                                         drm 13/09/90

: set-time      \ hours mins secs --
  rd-rtc
  0 !rtc-img  2 !rtc-img  4 !rtc-img
  rtc-img wr-rtc  ;

: set-date      \ day month year --
  rd-rtc
  10 !rtc-img  8 !rtc-img  6 !rtc-img
  rtc-img wr-rtc  ;

: set-dow       \ day-of-week --
  rd-rtc  12 rtc-i!  rtc-img wr-rtc  ;


End-Up-Load
