\ 8096/80C196 ROM PowerForth - Multi-Tasker & Int

pto

Copyright (c) 1988..1991 to:

MicroProcessor Engineering Limited
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441   ( international )
tel: 0703 631441        ( domestic )

September 1988: dynamic task reallocation and INIT-TASK added
October 1988  : page switching bullet-proofed, SELF returns tsk#
October 1991  : scheduler converted to testing all bits in task status byte

\ normal register equates

\ 0   constant r0                 \ register 0 always returns 0
\ 018 constant SP                 \ data stack pointer
\ 01A constant RP                 \ return stack pointer
\ PLM style register set - allows mixed language programming
\ 01C constant AX         01C constant AL   01D constant AH
\ 01E constant BX         01E constant BL   01F constant BH
\ 020 constant CX         020 constant CL   021 constant CH
\ 022 constant DX         022 constant DL   023 constant DH
\ 024 constant IP                 \ Forth Interpretive pointer
\ 026 constant UP                 \ user area pointer
\ 028 constant TOS                \ top of data stack register
\   028 constant tos.l            \ NB TOS is LONG aligned
\   029 constant tos.h
\ 02A constant tosext             \ TOS extension


\ Task Control Block (TCB) layout                  sfp 12/01/89

\  + 0,1  - SSP   Saved Parameter Stack Pointer
\  + 2    - STAT  BIT 7   1 = running, 0 = halted
\                 BIT 6   1 = message pending
\                 BIT 5   1 = event triggered
\  + 3    - TASK  Task # that sent message here
\  + 4,5  - MESG  Message address
\  + 6,7  - ACTw  CFA of word run as task
\  + 8,9  - EVNTw CFA of word run as event handler


\
\ offsets within the TCB save area
\
decimal
  0 equ tcbsp                   \ task's SP
  2 equ tcbst                   \ task status byte
  3 equ tcbid                   \ id of message sender
  4 equ tcbmsg                  \ task message data
  6 equ tcbaction               \ CFA of main word
  8 equ tcbevent                \ CFA of event word

  10 equ tcblen                 \ length of task control block

\
\ Bit mask equates for bits in the task status byte
\
hex
080 equ runmask			\ bit 7 = task running bit
07F equ /runmask
  7 equ runbit
040 equ msgmask			\ bit 6 = message received bit
0BF equ /msgmask
  6 equ msgbit
020 equ evtrigmask		\ bit 5 = event triggered but not run
0DF equ /evtrigmask
  5 equ evtrigbit
010 equ evrunmask		\ bit 4 = event run
0EF equ /evrunmask
  4 equ evrunbit

\
\ reserve RAM space for variables and TCBS         sfp 13/01/89
\
there constant tcbs
  tcblen #tasks * allot-ram     \ reserve TCB space
there equ endtcbs

\ pause

code pause	\ -- ; the scheduler
l: [schedule]
  di
  jbs task# 7 xschedule			\ single or multi ?
\
\ save previously running task ip, rp, up, sp
\
  push RP
  push AX   push BX   push CX   push DX \ PLM register set
  push IP   push UP
  push TOS   push TOSEXT
  st sp tcbsp curtask ,[]               \ save SP in TCB
\
\ select next task to run
\
l: [schedule]1
    incb task#                          \ step task I.D.
    add curtask # tcblen                \ step task TCB pointer
    cmp curtask # endtcbs               \ at end of TCBs ?
    e, if,  clrb task#                  \ yes, reset task no.
            ld curtask # tcbs           \  and task pointer
       endif,                           \
    ldb al tcbst curtask ,[]            \ running if any bit in TCBST set
    orb al al
  je [schedule]1			\ check task is running
\
\ run selected task - sp, up, rp, ip
\
  ld sp tcbsp curtask ,[]               \ restore SP from TCB
  pop tosext   pop tos                  \ restore Forth machine
  pop up   pop ip
  pop dx   pop cx   pop bx   pop ax
  pop rp

  ei

\
\ test if event flag for this task has been set
\
  ldb al tcbst curtask ,[]              \ get task status
  jbc al evtrigbit xschedule		\ skip if flag not set
  andb al # /evtrigmask			\ reset event trigger flag
  orb al # evrunmask			\ set event run flag
  stb al tcbst curtask ,[]              \ reset event bit
  ld ax tcbevent curtask ,[]   br ax [] \ get event, go do it

l: xschedule
  ei
  next,                                 \ back to Forth
end-code

\ self status

hex

code self       \ -- task# ; return current task#
  push tos                              \ save old TOS
  ldbze tos task#   andb tos.l # 07F    \ fetch and mask task#
  next,   end-code

code status	\ -- n ; run bit is masked out
  push tos				\ save old TOS
  ldbze tos tcbst curtask ,[]		\ get task status
  andb tos.l # 07F			\ mask top bit
  next,   end-code


\ wait-event/msg wait stop

: wait-event/msg        \ -- ; wait for message or event trigger
  begin  pause  status until  ;		\ wait for event, message, or user

: wait          \ n -- ; reschedule n times, testing for events
  ?dup                                  \ 0 = for ever
  if  0
      do  pause  loop			\ spin n times
  else  wait-event/msg
  endif  ;

: stop		\ -- ; put oneself to sleep
  self halt pause  ;			\ will return when RESTARTed


\ single multi                                     sfp 12/01/89

code single     \ -- ; disable scheduler by setting flag
  orb task# # 080   next,
end-code

code multi      \ -- ; enable  scheduler by clearing flag
  andb task# # 07F   next,
end-code


\ restart halt

code restart    \ task# -- ; mark task TCB as running
  mulub tos.l # tcblen                  \ form task offset
  add tos # tcbs tcbst +                \ add base address
  ldb al tos []                         \ get status byte
  orb al # runmask			\ set bit 7
  stb al tos []                         \
  pop tos   next,                       \ restore TOS
end-code

code halt       \ task# -- ; reset running bit in TCB
  mulub tos.l # tcblen                  \ form task offset
  add tos # tcbs tcbst +                \ add base address
  ldb al tos []                         \ get status byte
  andb al # /runmask			\ clear bit 7
  stb al tos []                         \
  pop tos   next,                       \ restore TOS
end-code


\ set-event ?event

code set-event  \ task# -- ; set task's event flag
  mulub tos.l # tcblen                  \ form task offset
  add tos # tcbs tcbst +                \ add base address
  ldb al tos []                         \ get status byte
  orb al # evtrigmask			\ set bit 5
  stb al tos []                         \
  pop tos   next,                       \ restore TOS
end-code

code clr-event-run	\ -- ; clear own task's event run flag
  ldb al tcbst curtask ,[]		\ get task status
  andb al # /evrunmask			\ clear status bit
  stb al tcbst curtask ,[]		\ store task status
  next,   end-code

code ?event     \ -- ; run event if flag set
  ldb al tcbst curtask ,[]              \ get task status
  jbc al evtrigbit qeventx		\ skip if flag not set
  andb al # /evtrigmask			\ reset event trigger flag
  orb al # evrunmask			\ set event run flag
  stb al tcbst curtask ,[]              \ store flags back
  ld ax tcbevent curtask ,[]   br ax [] \ get event, go do it
l: qeventx
  next,   end-code


\ msg? event?

HEX

code msg?       \ task# -- flag ; true if task has message
  mulub tos.l # tcblen                  \ form task offset
  add tos # tcbs tcbst +                \ add base address
  ldb al tos []   jbc al msgbit msg?1	\ false if flag clear
  ld tos # -1   next,                   \ return true flag
l: msg?1
  clr tos   next,                       \ return false flag
end-code

code event?     \ task# -- flag ; true if task had event
  mulub tos.l # tcblen                  \ form task offset
  add tos # tcbs tcbst +                \ add base address
  ldb al tos []				\ fetch task status byte
  jbc al evrunbit event?1		\ false if flag clear
  ld tos # -1   next,                   \ return true flag
l: event?1
  clr tos   next,                       \ return false flag
end-code


\ to-task to-event                                sfp 13/01/89

HEX

: to-task	\ cfa task# -- ; sets action of task when activated
  tcblen * tcbs + tcbaction + !  ;      \ set into TCB of task

: to-event	\ cfa event# -- ; sets action of event handler when triggered
  tcblen * tcbs + tcbevent + !  ;       \ set into TCB of task
\ (send-msg) (get-message)

hex

code (send-msg) \ addr task# -- ; store message in task
  mulub tos.l # tcblen                  \ offset in TCB area
  add tos # tcbs                        \ form address
  pop tcbmsg tos ,[]                    \ store message
  ldb al task#				\
  and al # 07F				\
  stb al tcbid tos ,[]			\
  ldb al tcbst tos ,[]                  \ load status byte
  orb al # 040                          \ set message bit
  stb al tcbst tos ,[]                  \ put status byte back
  pop tos                               \ restore TOS
  next,                                 \ all done
end-code

code (get-message)      \ -- addr task#
  push tos                              \ save old TOS
  push tcbmsg curtask ,[]               \ get message
  ldbze tos tcbid curtask ,[]           \ get task#
  ldb al tcbst curtask ,[]		\ load status byte
  andb al # 0BF				\ reset message bit
  stb al tcbst curtask ,[]		\ put status byte back
  next,   end-code


\ send-message get-message                         sfp 05/02/89

HEX

: send-message  \ addr task# -- ; send message to task
  begin  dup msg?		\ if task already has a message
    while  pause                \ wait
  repeat                        \ and try again
  (send-msg)  ;                 \ send the message

: get-message   \ -- addr task# ; wait for any message
  begin  self msg? 0=           \ check for a pending message
    while  pause                \ wait for it
  repeat                        \ try again
  (get-message)  ;              \ read it

\ activate init-multi

hex

: activate	\ task# -- ; Initialise a TCB, and start task running
  dup tcblen * tcbs + >r                \ -- task#
  per-task * init-u0 +			\ -- task-area
  task-init-s0 over +  012 -            \ form SP with 9 items
  dup  r@ tcbsp + !                     \  and set task's SP
\ -- task-area task-sp
  2dup 4 + !                            \ put UP at SP+4
  over task-init-r0 +  over 010 + !     \ put RP at SP+16
  r@ tcbaction +  over 6 + !            \ put IP at SP+6
  2drop					\ discard task and sp
  runmask r> tcbst + c!			\ set task status to running
  pause  ;				\ give scheduler a chance

: init-multi    \ -- ; initialisation with multi-tasking
  single				\ disable multitasking
\
\ Set task user areas to a defined state
\
  [ init-u0 per-task + ] literal	\ skip task 0
  [ #tasks per-task * ] literal  bounds	\  but include interrupt task area
  do  i per-task erase			\ clear space
      init-u0 i 040 cmove               \ reset user areas
  per-task +loop			\ in each task
\
\ initialise the task control blocks, but only task 0 needs to be set up
\
  tcbs endtcbs over - erase		\ clear task control blocks
  080 tcbs tcbst + c!			\ mark task 0 as running
  ['] cold tcbs tcbaction + !		\ set action of task 0 to COLD
  tcbs curtask !                        \ current task pointer
  080 task# c!                          \ current task is 0, tasker disabled
  0 restart  ;				\ task 0 is Forth
\ multi-tasking version of key

\
\ In order for the Forth system to use a cooperative scheduler, PAUSE or
\ WAIT must be executed periodically. A natural place to insert PAUSE is
\ when polling the keyboard, or waiting for any other I/O to complete.
\
: mt-key        \ -- char ; schedules while waiting for character
  Begin  (key?) 0=                      \ while no character
    While  pause                        \ call scheduler
  Repeat                                \
  (key)  ;                              \ now get it

  assign mt-key to-do key               \ switch as action of KEY
\ semaphore

\
\ A SEMAPHORE is an extended variable used for signalling between tasks,
\ and for resource allocation.
\ The counter field is used as a count of the number of times the
\ resource may be used, and the arbiter field contains the TCB of the
\ task that last gained access. This field can be used for priority
\ arbitration and deadlock detection/arbitration.
\
: semaphore	\ -- ; -- addr [child]
  create  there ,			\ point to RAM
          0 ,(r)  0 ,(r)		\ count and arbiter fields
  does> @  ;

\
\ SIGNAL increments the counter field of a semaphore, indicating
\ either a) that another item has been allocated to the resource, or
\        b) it is available for use again, 0 indicating in use by a task
\
: signal	\ addr -- ; increment counter field of semaphore,
		\ so making it available for use by a task
  di  1 over +!  ei  ;			\ inc. counter

\
\ RELEASE is for use in REQUEST/RELEASE pairs for critical regions.
\ Because the cross compiler initialises data to zero, all semaphores
\ should be RELEASEd before use in the initialisation code.
\
: release	\ addr -- ; set count field to one, and zero the owner field
  di  1 over !  2+ off  ei  ;		\ set count to one, mark not owned
\
\ REQUEST waits until the counter field of a semaphore is non-zero,
\ and then decrements the counter field by one.
\ This allows the semaphore to be used as a COUNTED semaphore. For
\ example a character buffer may be used where the semaphore counter
\ shows the number of available characters.
\ Alternatively the semaphore may be used purely to share resources.
\ The semaphore is initialised to one. The first task to REQUEST it
\ gains access, and all other tasks must wait until the accessing task
\ SIGNALs that it has finished with the resource, or RELEASEs it.
\
: request	\ addr -- ; get access to resource, wait if count = 0
  begin
    dup  di  @ 0=                       \ n.b test and set
    while  ei pause                     \ operations must be
  repeat                                \ non-interruptible and indivisible
  -1 over +!                            \ got it, decrement counter
  curtask @ swap 2+ !			\ mark resource as mine
  ei  ;					\ re-enable interrupts
\ all done

decimal

;p


