\ Target block words for use with XSHELL v2

((
Copyright (c) 1991

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

First Release: 29/04/91

Changes:	See RELEASE.DOC for full details

Last release: 26/01/90
))

\ blk-read blk-write

internal

decimal

: BLK-READ	\ addr blk# -- ; reads one block from host to addr
  ^R ser-emit                   \ send read code
  synch-to-host                 \ synchronise host & target
  send-block#                   \ send block no.
  1024 bounds                   \ formulate buffer addresses
  DO
    ser-key i c!                \ get 8 bits, save in buffer
  LOOP
  ACK ser-emit                  \ send ack to complete transfer
  Wait-Ack/Nack Disk-error !    \ get status response
; 

: BLK-WRITE	\ addr blk# -- ; Sends given block number to host
  ^W ser-emit                   \ send write code
  synch-to-host                 \ synchronise host & target
  send-block#                   \ send block no.
  1024 bounds                   \ formulate buffer addresses
  DO
    I C@ ser-emit               \ send 8 bits from buffer
  LOOP
  Wait-Ack/Nack Disk-error !    \ get status response
; 


\ user vars and block buffer                       sfp 16/09/90

hex

variable disk-error     \ -- addr ; set non-zero on error

VARIABLE FIRST          \ -- addr ; first element is block no.
  07FFF FIRST !                 \ set to improbable value
  rom? if(  0400 ALLOT-RAM  )else(  0400 allot  )endif

external


\ save-buffers                                     sfp 16/09/90

HEX

: SAVE-BUFFERS	\ -- ; sabe buffer if modified
  FIRST @ 0<                  \ buffer marked as changed?
  IF                          \ if so....
    FIRST 2+                  \   leave buffer address
    FIRST @ 07FFF AND         \   clear update flag
    DUP FIRST !               \   buffer-addr blk# --
    BLK-WRITE                 \   write this changed block
  endif                       \ or else don't save it
; 


\ empty-buffers                                    sfp 16/09/90

HEX

: EMPTY-BUFFERS	\ -- ; mark buffer as empty
  FIRST 0402 ERASE            \ clear buffer to zeroes
  07FFF FIRST !               \ clear resident block number
;


\ update flush                                     sfp 16/09/90

HEX

: UPDATE                \ -- ;
  FIRST @  08000 OR            \
  FIRST !                      \ mark resident block as changed
;

: FLUSH                 \ -- ;
  SAVE-BUFFERS  EMPTY-BUFFERS  \ flush any changes to disk
;
\ block                                            sfp 04/09/90
HEX

: (BLOCK)       \ blk# -- address ;
  DUP  FIRST @ 07FFF AND  =     \ already in ?
  IF
    DROP                        \ yes, discard
  ELSE
    SAVE-BUFFERS                \ no, save current buffer
    FIRST 2+ OVER BLK-READ      \ read new
    FIRST !                     \ set blk#
  ENDIF                         \
  FIRST 2+  ;                   \ return addr (fixed)

assign (block) to-do block


\ cls (line)                                    R. sfp 26/01/90

hex

: CLS                   \ -- ;
  03 ser-emit                 \ send a ^C to trigger host CLS
;

internal

: (LINE)	\ line# blk# -- addr #chars ;
  >R 040 0400 */MOD R> + 
  BLOCK + 040                 \ step line by line through block
;                             \ buffer


\ -trailing .line ?loading

hex
\
\ This word strips the trailing spaces from a string. It is most useful when
\ displaying lines from a screen to strip the trailing spaces from the
\ 64 byte fixed length lines. This word is also provided as part of
\ TOOLS.FTH.
\
: -trailing	\ addr len -- addr len' ; strip trailing spaces
   dup 0
   ?do  2dup + 1- c@ bl <> ?leave 1-  loop  ;

: .LINE		\ line# blk# -- ;
  (LINE)  -TRAILING  TYPE       \ show contents of line
;

: ?LOADING	\ -- ; error if input from console
  BLK @  0=                     \ BLK==0 : console
  016 ?ERROR                    \ error if not loading
;

external
\ list                                          R. sfp 26/01/90

hex

: LIST		\ blk# -- ; display screen given
  CLS DUP SCR !                 \ clear screen, set variable
  ." Screen " .                 \ print header
  010 0                         \ the number of lines
  DO
    CR  I 1+ 3 .R SPACE         \ for each line print line no.
    I SCR @ .LINE               \ and line from block
    KEY? IF  LEAVE  endif       \ check if early finish
  LOOP
  CR                            \ new line
;


\ Misc Output .....................................sfp 05/09/90

hex

: L             \ --
  Scr @ List  ;

: N             \ --
  Scr @ 1+  07FFF and  List  ;

: P             \ --
  Scr @ 1-  0 max  List  ;

Decimal

: Index         \ n1 n2 -- ; list top lines in range of screens
  Cr  1+ Swap
  ?DO  I Block Drop
       Halt? Disk-Error @ Or ?LEAVE
       Cr I 3 .R Space  0 I .Line
  LOOP  ;

: Qx            \ --
  0 32767 Index  ;
\ load thru -->

hex

: LOAD		\ blk# -- ; compile given screen
  BLK @ >R  >IN @ >R            \ preserve blk# & input position
  0 >IN !  BLK !                \ set up for this block number
  ?LOADING                      \ check for block 0
  INTERPRET                     \ interpret from this block
  R> >IN !  R> BLK !            \ finished: restore pointers
;

: THRU                  \ blk#from blk#to -- ;
  1+ SWAP                       \ form limit & index
  DO
    I LOAD                      \ load the index number block
  LOOP
;

: -->                   \ -- ;
  ?LOADING                      \ must be loading
  0 >IN !  1 BLK +!             \ step to next block
; immediate                     \ this word is immediate:
                                \ a word may cover 2+ screens

\ These words provide control within a file, same as host words


\ $using using                                  R. sfp 26/01/90

: $using                \ $addr -- ;
  ^u send$                      \ send string to host
;

: using                 \ <file> -- ;
  bl word                       \ get string from keyboard
  $using                        \ open it on host
;

\ NB $USING in this implementation does not return a t/f flag
\ as the host version does. The host takes care of creating
\ non existent files.
\ all done

decimal

;p
