((
\ high level floating point code file - ROM model

Copyright (C) 1983-1990
MicroProcessor Engineering Ltd.
133 Hill Lane
Shirley
Southampton SO1 5AF
England
tel: 0703 631441
))

\ ftemp f! f@ fdup fover frot fswap fdrop

((
    Many of the floating pointed operations run faster when implemented
    using a temporary buffer in RAM. This is particularly true of the RTX
    processor when using a common data stack for floating point and
    integer numbers.
))

decimal

Variable ftemp  18 allot-ram  \ -- addr ; three item buffer

: f!		\ fn addr -- ; note top of stack to low address
  >r  r@ !  r@ 2+ !  r> 4 + !  ;

: f@		\ addr -- fn ; top of stack from low address
  >r  r@ 4 + @  r@ 2+ @  r> @  ;

: fdup		\ fn -- fn fn
  ftemp f!  ftemp f@  ftemp f@  ;

: fover		\ f1 f2 -- f1 f2 f1
  [ ftemp 6 + ] literal f!  ftemp f!
  ftemp f@  [ ftemp 6 + ] literal f@  ftemp f@  ;

decimal

: frot		\ f1 f2 f3 -- f2 f3 f1
  ftemp f!				\ save f3 in buffer
  [ ftemp 6 + ] literal f!		\ save f2 in buffer
  [ ftemp 12 + ] literal f! 		\ save f1 in buffer
  [ ftemp 6 + ] literal f@		\ f2
  [ ftemp ] literal f@			\ f3
  ftemp 12 + f@  ;			\ f1

: fswap		\ f1 f2 -- f2 f1
  ftemp f!  [ ftemp 6 + ] literal f!
  ftemp f@  [ ftemp 6 + ] literal f@  ;

: fdrop		\ f1 --
  drop drop drop  ;

\ 8086 shift routines .............................sfp 20/11/89

hex

: <-S		\ d1 carry-in-flag -- d2 carry-out-flag
  -rot				\ save flag away
  dup 0< >r			\ save carry out
  2dup d+			\ shift left by doubling
  rot if  0,1 d+  endif		\ add carry in
  r> 1 and  ;			\ provide carry out

: S->		\ d1 carry-in-flag -- d2 carry-out-flag
  >r				\ save carry in flag
  over 1 and  -rot		\ save carry out flag
  swap 2/ 07FFF and		\ shift right l.s. word
  over 1 and			\ look at carry from m.s. word
  if  08000 or  endif		\ merge into l.s. word
  swap 2/ 07FFF and		\ shift right m.s. word
  r>				\ look at carry in
  if  08000 or  endif		\ merge into m.s. word
  rot  ;			\ bring back carry out

: d<<1		\ d1 -- d1<<1
  2dup d+  ;

: d>>1		\ d1 -- d1>>1
  swap 2/ 07FFF and		\ shift right l.s. word
  over 1 and			\ look at carry from m.s. word
  if  08000 or  endif		\ merge into l.s. word
  swap 2/ 07FFF and  ;		\ shift right m.s. word

: q<<1		\ q1 -- q2
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*				\ shift lowest
  r> r> r>  ;			\ restore higher items

decimal
\ defining words

: fvariable	( -- ; -- addr )
  CREATE  here 2+ ,  0 , 0 , 0 ,  DOES> @  ;

: fconstant	( fn -- )
  CREATE  , , ,  DOES>  F@  ;

: farray	( fn-1..f0 n -- ; compile / n -- fn ; execute )
  CREATE  0
          DO  , , ,  LOOP
  DOES>  SWAP 6 * +  F@  ;

;p
